package de.javafrog.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

/**
 * @author Javafrog
 *
 */

public class GuiUtil
{
public static void centerWindowOnScreen(Component window)
  {
   window.setLocation(getScreenCenterX()-Math.round(window.getWidth()/2),getScreenCenterY()-Math.round(window.getHeight()/2));
  }
  
public static void centerWindowOnParent(Component window,Component parent)
  {
   window.setLocation(Math.max(0,getWindowCenterX(parent)-Math.round(window.getWidth()/2)),Math.max(0,getWindowCenterY(parent)-Math.round(window.getHeight()/2)));
  }
  
public static int getWindowCenterX(Component window)
  {
   return window.getX()+Math.round(window.getWidth()/2);
  }
  
public static int getWindowCenterY(Component window)
  {
   return window.getY()+Math.round(window.getHeight()/2);
  }

public static int getScreenCenterX()
  {
   return Math.round(Toolkit.getDefaultToolkit().getScreenSize().width/2);
  }
  
public static int getScreenCenterY()
  {
   return Math.round(Toolkit.getDefaultToolkit().getScreenSize().height/2);
  }

public static void resizeToMaxDimension(JComponent[] objects)
  {
   int xMax = 0;
   int yMax = 0;
    
   for (int i = 0; i < objects.length; i++)
     {
      xMax = Math.max(xMax,objects[i].getPreferredSize().width);
      yMax = Math.max(yMax,objects[i].getPreferredSize().height);
     }
   
   for (int i = 0; i < objects.length; i++)
     {
      objects[i].setPreferredSize(new Dimension(xMax,yMax));
     }
  }

public static ImageIcon createImageIcon(String path,String description)
  {
   URL imgURL = GuiUtil.class.getResource(path);
   
   if (imgURL != null) return new ImageIcon(imgURL, description);

   System.err.println("Couldn't find file: " + path);
   return null;
  }

public static void addFileChooser(final JButton triggerButton,final JTextField resultField)
  {
   addFileChooser(triggerButton,resultField,null);
  }

public static void addFileChooser(final JButton triggerButton,final JTextField resultField,final FileFilter filter)
  {
   triggerButton.addActionListener(new ActionListener()
     {
      @Override
      public void actionPerformed(ActionEvent e)
        {
         JFileChooser jfc = new JFileChooser();
         
         if (filter != null) jfc.setFileFilter(filter);
         
         int returnVal = jfc.showOpenDialog(triggerButton);
         
         if (returnVal == JFileChooser.APPROVE_OPTION)
           {
            File file = jfc.getSelectedFile();
            resultField.setText(file.getPath());
           }
        }      
     });
  }
}