package de.mmt.mpeg;

import de.mmt.mpeg.framework.data.MacroBlock;

/**
 * Helper class for Exercise 3.3
 * 
 * @author MaNi
 */

public class Exercise3Helper
{
/**
 * Creates a difference block from an existing macro block and another block from a previous frame
 * at a different offset position.
 * To later restore the original block, the compared block is required again.
 *  
 * @param original the original block, which should be encoded as a difference block
 * @param comparedBlock the block against which the original block is coded
 * @param offsetX the horizontal offset of the compared block
 * @param offsetY the vertical offset of the compared block
 * 
 * @return a difference-coded representation of the original macro block
 */

public static MacroBlock createDifferenceBlock(MacroBlock original,MacroBlock comparedBlock,int offsetX,int offsetY)
  {
   MacroBlock diffBlock = new MacroBlock(original);
   
   // if the compared block is null, create a copy of the original block
   if (comparedBlock == null) return diffBlock;

   diffBlock.setOffsetX(offsetX);
   diffBlock.setOffsetY(offsetY);
   diffBlock.setDifferenceCoded(true);      
   
   for (int i = 0; i < MacroBlock.BLOCKSIZE; i++)
     {
      for(int j = 0; j < MacroBlock.BLOCKSIZE; j++)
        {
         diffBlock.setY(i,j,original.getY(i,j) - comparedBlock.getY(i,j));
        }
     }
   for (int i = 0; i < MacroBlock.BLOCKSIZE/MacroBlock.XFACTOR; i++)
     {
      for (int j = 0; j < MacroBlock.BLOCKSIZE/MacroBlock.YFACTOR; j++)
        {
         diffBlock.setCb(i,j,original.getCb(i,j) - comparedBlock.getCb(i,j));
         diffBlock.setCr(i,j,original.getCr(i,j) - comparedBlock.getCr(i,j));
        }
     }
   
   return diffBlock;
  }

/**
 * Returns the distance of two macro blocks. The distance is determined by adding up
 * the distance of all three layers: Y, Cb and Cr. The distance of each layer is determined
 * by the sum of all distances of individual pixel values.
 * Values may range from 0 (exact match) to 97920 (Y 255*16 + Cb 255*8 + Cr 255*8), with
 * 255 being the maximum distance for one layer value of a pixel.
 * 
 * @param mb1 one macro block to compare
 * @param mb2 the other macro block to compare
 * 
 * @return the absolute distance between the two macro blocks
 */

public static int computeDistance(MacroBlock mb1, MacroBlock mb2)
  {
   int sum = 0;
   
   // sum Y distances
   for (int i = 0; i < MacroBlock.BLOCKSIZE; i++)
     {
      for (int j = 0; j < MacroBlock.BLOCKSIZE; j++)
        {
         sum += Math.abs(mb2.getY(i,j) - mb1.getY(i,j));
        }
     }
   
   // sum Cb and Cr distances
   for (int i = 0; i < MacroBlock.BLOCKSIZE/MacroBlock.XFACTOR; i++)
     {
      for (int j = 0; j < MacroBlock.BLOCKSIZE/MacroBlock.YFACTOR; j++)
        {
         sum += Math.abs(mb2.getCb(i,j) - mb1.getCb(i,j));
         sum += Math.abs(mb2.getCr(i,j) - mb1.getCr(i,j));
        }
     }
   
   return sum;
  }
}