package de.mmt.mpeg.framework.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import de.javafrog.gui.GuiUtil;
import de.mmt.mpeg.framework.data.MacroBlock;
import de.mmt.mpeg.framework.data.MatrixTriple;
import de.mmt.mpeg.framework.util.BLKFilter;
import de.mmt.mpeg.framework.util.Util;

public class BlockRestorePanel extends JPanel
{
private static final long serialVersionUID = 5311701399279529809L;
private JButton loadbutton;
private JLabel imgarea;

public BlockRestorePanel()
  {
   this.setOpaque(false);
   this.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
   this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
   
   JPanel loadpanel = new JPanel(new BorderLayout(5,0));
   loadpanel.setOpaque(false);
   loadpanel.setMaximumSize(new Dimension(Integer.MAX_VALUE,20));
   
   JPanel filepanel = new JPanel(new BorderLayout(5,0));
   filepanel.setOpaque(false);
   final JTextField inputfile = new JTextField();
   JButton searchbutton = new JButton("...");
   searchbutton.setToolTipText("Durchsuchen...");
   GuiUtil.addFileChooser(searchbutton,inputfile,new BLKFilter());
   filepanel.add(inputfile,BorderLayout.CENTER);
   filepanel.add(searchbutton,BorderLayout.EAST);
   
   loadbutton = new JButton("Makroblockdatei ffnen");
   loadbutton.setToolTipText("Ldt das Bild und zeigt es im unteren Bereich an.");
   loadpanel.add(filepanel,BorderLayout.CENTER);
   loadpanel.add(loadbutton,BorderLayout.EAST);
   
   imgarea = new JLabel();
   JScrollPane imgscroller = new JScrollPane(imgarea);
   imgscroller.getVerticalScrollBar().setUnitIncrement(10);
   imgscroller.getHorizontalScrollBar().setUnitIncrement(10);
   imgscroller.setPreferredSize(new Dimension(500,360));  
   

   this.add(loadpanel);
   this.add(Box.createVerticalStrut(5));
   this.add(imgscroller);
   
   // load the image
   loadbutton.addActionListener(new ActionListener()
     {
      @Override
      public void actionPerformed(ActionEvent e)
        {
         String userfile = inputfile.getText();
         if (userfile == null || userfile.isEmpty()) return;
       
         File f = new File(userfile);
       
         if (!f.exists())
           {
            JOptionPane.showMessageDialog(loadbutton,"Die angegebene Datei '"+userfile+"' wurde nicht gefunden.","Fehler beim ffnen",JOptionPane.ERROR_MESSAGE);
            return;
           }

         try
           {
            MacroBlock[][] frame = Util.importBlockMatrix(f.getPath());
            MatrixTriple ycc;
         
            try
              {
               ycc = Util.restoreOriginalFrame(frame,null);
              }
            catch (IllegalArgumentException ex)
              {
               // a macro block requires a diff frame, let user specify
               JFileChooser jfc = new JFileChooser();
               jfc.setFileFilter(new BLKFilter());
               jfc.setDialogTitle("Differenzframe laden...");
               int returnVal = JFileChooser.ERROR_OPTION;
               File diffFrame = null;
               
               while (returnVal != JFileChooser.APPROVE_OPTION || diffFrame == null || diffFrame.isDirectory())
                 {
                  returnVal = jfc.showOpenDialog(null);
                  
                  // when user presses cancel, abort everything
                  if (returnVal == JFileChooser.CANCEL_OPTION)
                    {                  
                     return;
                    }
                  
                  diffFrame = jfc.getSelectedFile();
                 }
               
               MacroBlock[][] previousFrame = Util.importBlockMatrix(diffFrame.getPath());
               ycc = Util.restoreOriginalFrame(frame,previousFrame);
              }
            
            MatrixTriple rgb = Util.transformYCC2RGB(ycc);
            BufferedImage img = Util.createImageFromMatrixTriple(rgb);
            imgarea.setIcon(new ImageIcon(img));
           }
         catch (FileNotFoundException ex2)
           {
            ex2.printStackTrace();
           }
        }      
     });
  }
}