package de.mmt.mpeg.framework.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import de.javafrog.gui.GuiUtil;
import de.mmt.mpeg.framework.process.BlockSearchProcess;
import de.mmt.mpeg.framework.util.JPGFilter;

public class BlockSearchPanel extends JPanel
{
private static final long serialVersionUID = 5311701399279529809L;
private JButton loadbutton;
private JButton diffbutton;
private BufferedImage currentImage;
private JLabel imgarea;

public BlockSearchPanel()
  {
   this.setOpaque(false);
   this.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
   this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
   
   JPanel loadpanel = new JPanel(new BorderLayout(5,0));
   loadpanel.setOpaque(false);
   loadpanel.setMaximumSize(new Dimension(Integer.MAX_VALUE,20));
   
   JPanel filepanel = new JPanel(new BorderLayout(5,0));
   filepanel.setOpaque(false);
   final JTextField inputfile = new JTextField();
   JButton searchbutton = new JButton("...");
   searchbutton.setToolTipText("Durchsuchen...");
   GuiUtil.addFileChooser(searchbutton,inputfile,new JPGFilter());
   filepanel.add(inputfile,BorderLayout.CENTER);
   filepanel.add(searchbutton,BorderLayout.EAST);
   
   loadbutton = new JButton("Bild ffnen");
   loadbutton.setToolTipText("Ldt das Bild und zeigt es im unteren Bereich an.");
   loadpanel.add(filepanel,BorderLayout.CENTER);
   loadpanel.add(loadbutton,BorderLayout.EAST);
   
   imgarea = new JLabel();
   JScrollPane imgscroller = new JScrollPane(imgarea);
   imgscroller.getVerticalScrollBar().setUnitIncrement(10);
   imgscroller.getHorizontalScrollBar().setUnitIncrement(10);
   imgscroller.setPreferredSize(new Dimension(500,360));
   
   JPanel runpanel = new JPanel(new FlowLayout(FlowLayout.RIGHT,0,0));
   runpanel.setOpaque(false);   
   diffbutton = new JButton("Differenzbild erzeugen",GuiUtil.createImageIcon("/images/control_play_blue.png",""));
   diffbutton.setToolTipText("Erzeugt aus dem Ausgangsbild und einem zweiten Frame einen Differenzframe, der als Makroblockdatei gespeichert werden kann.");
   diffbutton.setEnabled(false);
   runpanel.add(diffbutton);   

   this.add(loadpanel);
   this.add(Box.createVerticalStrut(5));
   this.add(imgscroller);
   this.add(Box.createVerticalStrut(5));
   this.add(runpanel);
   
   // load the image
   loadbutton.addActionListener(new ActionListener()
     {
      @Override
      public void actionPerformed(ActionEvent e)
        {
         String userfile = inputfile.getText();
         if (userfile == null || userfile.isEmpty()) return;
       
         File f = new File(userfile);
       
         if (!f.exists())
           {
            JOptionPane.showMessageDialog(loadbutton,"Die angegebene Datei '"+userfile+"' wurde nicht gefunden.","Fehler beim ffnen",JOptionPane.ERROR_MESSAGE);
            return;
           }

         try
           {
            BufferedImage img = ImageIO.read(f);
            imgarea.setIcon(new ImageIcon(userfile));
            
            // if its really an image
            if (img != null)
              {
               currentImage = img;
               diffbutton.setEnabled(true);
              }
           }
         catch (IOException ex)
           {
            ex.printStackTrace();
           }
        }      
     });
   
   // convert the image from rgb to ycbcr
   diffbutton.addActionListener(new ActionListener()
     {
      @Override
       public void actionPerformed(ActionEvent e)
        {
         diffbutton.setEnabled(false);
         loadbutton.setEnabled(false);
         final BlockSearchProcess process = new BlockSearchProcess(currentImage,new ActionListener()
           {
            @Override
            public void actionPerformed(ActionEvent e)
              {
               diffbutton.setEnabled(true);
               loadbutton.setEnabled(true);
               currentImage = ((BlockSearchProcess)e.getSource()).getCurrentImage();
               imgarea.setIcon(new ImageIcon(currentImage));
              }
            
           });
         process.start();         
        }
     });   
  }
}