package de.mmt.mpeg.framework.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import de.javafrog.gui.GuiUtil;
import de.mmt.mpeg.framework.process.MacroblockProcess;
import de.mmt.mpeg.framework.util.JPGFilter;

public class MacroblockPanel extends JPanel
{
private static final long serialVersionUID = 5311701399279529809L;
private JButton loadbutton;
private JButton convertbutton;
private JButton convertpixelbutton;
private BufferedImage currentImage;
private JLabel imgarea;

public MacroblockPanel()
  {
   this.setOpaque(false);
   this.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
   this.setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
   
   JPanel loadpanel = new JPanel(new BorderLayout(5,0));
   loadpanel.setOpaque(false);
   loadpanel.setMaximumSize(new Dimension(Integer.MAX_VALUE,20));
   
   JPanel filepanel = new JPanel(new BorderLayout(5,0));
   filepanel.setOpaque(false);
   final JTextField inputfile = new JTextField();
   JButton searchbutton = new JButton("...");
   searchbutton.setToolTipText("Durchsuchen...");
   GuiUtil.addFileChooser(searchbutton,inputfile,new JPGFilter());
   filepanel.add(inputfile,BorderLayout.CENTER);
   filepanel.add(searchbutton,BorderLayout.EAST);
   
   loadbutton = new JButton("Bild ffnen");
   loadbutton.setToolTipText("Ldt das Bild und zeigt es im unteren Bereich an.");
   loadpanel.add(filepanel,BorderLayout.CENTER);
   loadpanel.add(loadbutton,BorderLayout.EAST);
   
   imgarea = new JLabel();
   JScrollPane imgscroller = new JScrollPane(imgarea);
   imgscroller.getVerticalScrollBar().setUnitIncrement(10);
   imgscroller.getHorizontalScrollBar().setUnitIncrement(10);
   imgscroller.setPreferredSize(new Dimension(500,360));
   
   JPanel runpanel = new JPanel(new FlowLayout(FlowLayout.RIGHT,0,0));
   runpanel.setOpaque(false);   
   convertbutton = new JButton("Makroblcke erzeugen",GuiUtil.createImageIcon("/images/control_play_blue.png",""));
   convertbutton.setToolTipText("Konvertiert das Bild mittels Ihres Algorithmus in das Makroblockformat. Zu Testzwecken wird das Bild anschlieend zurckgewandelt und oben angezeigt.");
   convertbutton.setEnabled(false);
   convertpixelbutton = new JButton("Makroblcke erzeugen (Mosaikansicht)",GuiUtil.createImageIcon("/images/control_play_blue.png",""));
   convertpixelbutton.setToolTipText("hnlich wie die normale Makroblockerzeugung, visualisiert aber zustzlich die generierten Makroblcke als Mosaik.");
   convertpixelbutton.setEnabled(false);
   runpanel.add(convertpixelbutton);   
   runpanel.add(Box.createHorizontalStrut(5));
   runpanel.add(convertbutton);   

   this.add(loadpanel);
   this.add(Box.createVerticalStrut(5));
   this.add(imgscroller);
   this.add(Box.createVerticalStrut(5));
   this.add(runpanel);
   
   // load the image
   loadbutton.addActionListener(new ActionListener()
     {
      @Override
      public void actionPerformed(ActionEvent e)
        {
         String userfile = inputfile.getText();
         if (userfile == null || userfile.isEmpty()) return;
       
         File f = new File(userfile);
       
         if (!f.exists())
           {
            JOptionPane.showMessageDialog(loadbutton,"Die angegebene Datei '"+userfile+"' wurde nicht gefunden.","Fehler beim ffnen",JOptionPane.ERROR_MESSAGE);
            return;
           }

         try
           {
            BufferedImage img = ImageIO.read(f);
            imgarea.setIcon(new ImageIcon(userfile));
            
            // if its really an image
            if (img != null)
              {
               currentImage = img;
               convertbutton.setEnabled(true);
               convertpixelbutton.setEnabled(true);
              }
           }
         catch (IOException ex)
           {
            ex.printStackTrace();
           }
        }      
     });
   
   // convert the image from rgb to ycbcr
   convertbutton.addActionListener(new ActionListener()
     {
      @Override
       public void actionPerformed(ActionEvent e)
        {
         convertbutton.setEnabled(false);
         convertpixelbutton.setEnabled(false);
         loadbutton.setEnabled(false);
         final MacroblockProcess process = new MacroblockProcess(currentImage,new ActionListener()
           {
            @Override
            public void actionPerformed(ActionEvent e)
              {
               if (e.getActionCommand().equals("error")) JOptionPane.showMessageDialog(loadbutton,"Es ist ein Fehler aufgetreten.","Verarbeitungsfehler",JOptionPane.ERROR_MESSAGE);
               
               convertbutton.setEnabled(true);
               convertpixelbutton.setEnabled(true);
               loadbutton.setEnabled(true);
               currentImage = ((MacroblockProcess)e.getSource()).getCurrentImage();
               imgarea.setIcon(new ImageIcon(currentImage));
              }
            
           });
         process.start();         
        }
     });
   
   // convert the image from rgb to ycbcr
   convertpixelbutton.addActionListener(new ActionListener()
     {
      @Override
       public void actionPerformed(ActionEvent e)
        {
         convertbutton.setEnabled(false);
         convertpixelbutton.setEnabled(false);
         loadbutton.setEnabled(false);
         final MacroblockProcess process = new MacroblockProcess(currentImage,new ActionListener()
           {
            @Override
            public void actionPerformed(ActionEvent e)
              {
               if (e.getActionCommand().equals("error")) JOptionPane.showMessageDialog(loadbutton,"Es ist ein Fehler aufgetreten.","Verarbeitungsfehler",JOptionPane.ERROR_MESSAGE);
               
               convertbutton.setEnabled(true);
               convertpixelbutton.setEnabled(true);
               loadbutton.setEnabled(true);
               currentImage = ((MacroblockProcess)e.getSource()).getCurrentImage();
               imgarea.setIcon(new ImageIcon(currentImage));
              }
            
           },true);
         process.start();
        }
     });
  }
}