package de.mmt.mpeg.framework.process;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;

import de.mmt.mpeg.Exercise2;
import de.mmt.mpeg.Exercise3;
import de.mmt.mpeg.framework.data.MacroBlock;
import de.mmt.mpeg.framework.data.MatrixTriple;
import de.mmt.mpeg.framework.util.BLKFilter;
import de.mmt.mpeg.framework.util.JPGFilter;
import de.mmt.mpeg.framework.util.Util;

public class BlockSearchProcess extends Thread
{
private ActionListener finalAction = null;
private BufferedImage currentImage = null;

public BlockSearchProcess(BufferedImage imageToConvert,ActionListener finalAction)
  {
   this.finalAction = finalAction;
   this.currentImage = imageToConvert;
  }

public BufferedImage getCurrentImage()
  {
   return this.currentImage;
  }

@Override
public void run()
  {
   Exercise3 algorithm = new Exercise3();
   Exercise2 blockMaker = new Exercise2();
   
   try
     {
      JFileChooser jfc = new JFileChooser();
      jfc.setDialogTitle("Differenzframe laden...");
      jfc.setFileFilter(new JPGFilter());
      int returnVal = JFileChooser.ERROR_OPTION;
      File diffFrame = null;
      
      while (returnVal != JFileChooser.APPROVE_OPTION || diffFrame == null || diffFrame.isDirectory())
        {
         returnVal = jfc.showOpenDialog(null);
         
         // when user presses cancel, abort everything
         if (returnVal == JFileChooser.CANCEL_OPTION)
           {
            if (this.finalAction != null) this.finalAction.actionPerformed(new ActionEvent(this,1,"aborted"));
            return;
           }
         
         diffFrame = jfc.getSelectedFile();
        }

      MatrixTriple rgb = Util.getRGBMatricesFromImage(this.currentImage);   
      BufferedImage img = ImageIO.read(diffFrame);
      MatrixTriple rgb2 = Util.getRGBMatricesFromImage(img);
      

      MacroBlock[][] mbm = blockMaker.generateMacroBlockMatrix(rgb);
      MacroBlock[][] mbm2 = blockMaker.generateMacroBlockMatrix(rgb2);
      
      MacroBlock[][] mbmDiff = algorithm.generateDifferenceMacroBlockMatrix(mbm,mbm2);      
      
      jfc = new JFileChooser();
      jfc.setDialogTitle("Differenz-Makroblockdatei speichern...");
      jfc.setFileFilter(new BLKFilter());
      returnVal = JFileChooser.ERROR_OPTION;
      File storageFile = null;
      
      while (returnVal != JFileChooser.APPROVE_OPTION || storageFile == null || storageFile.isDirectory())
        {
         returnVal = jfc.showSaveDialog(null);
         
         // when user presses cancel, abort everything
         if (returnVal == JFileChooser.CANCEL_OPTION)
           {
            if (this.finalAction != null) this.finalAction.actionPerformed(new ActionEvent(this,1,"aborted"));
            return;
           }
         
         storageFile = jfc.getSelectedFile();
        }
      
      if (!storageFile.getName().toLowerCase().endsWith(".blk")) storageFile = new File(storageFile.getParent(),storageFile.getName()+".blk");
      
      // save macroblock file
      Util.exportBlockMatrix(mbmDiff,storageFile.getPath());           
      
      //D:\Users\mani\Desktop\Uebung_MMS\2011\Aufgabe 3 - MPEG\Material\DPP_0001.JPG
      
      // restore original image format
      MatrixTriple ycc = Util.restoreOriginalFrame(mbmDiff,mbm2);      
      //System.out.println("### Restored frame ###");
      
      // back to RGB
      MatrixTriple restoredRGB = Util.transformYCC2RGB(ycc);
      BufferedImage restoredImg = Util.createImageFromMatrixTriple(restoredRGB);

      //System.out.println("### Setting RGB image on canvas ###");
      this.currentImage = restoredImg;
     }
   catch (Exception e)
     {
      e.printStackTrace();
     }
   
   if (this.finalAction != null) this.finalAction.actionPerformed(new ActionEvent(this,1,"completed"));
  }
}