package de.mmt.mpeg.framework.process;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

import de.mmt.mpeg.Exercise1;
import de.mmt.mpeg.framework.data.MatrixTriple;
import de.mmt.mpeg.framework.util.Util;

public class ConversionProcess extends Thread
{
private boolean doGrayScaling = false;
private ActionListener finalAction = null;
private BufferedImage currentImage = null;

public ConversionProcess(BufferedImage imageToConvert,ActionListener finalAction)
  {
   this(imageToConvert,finalAction,false);
  }

public ConversionProcess(BufferedImage imageToConvert,ActionListener finalAction,boolean doGrayScaling)
  {
   this.finalAction = finalAction;
   this.currentImage = imageToConvert;
   this.doGrayScaling = doGrayScaling;
  }

public BufferedImage getCurrentImage()
  {
   return this.currentImage;
  }

@Override
public void run()
  {
   Exercise1 algorithm = new Exercise1();
   
   try
     {
      MatrixTriple rgb = Util.getRGBMatricesFromImage(this.currentImage);
      MatrixTriple ycc = algorithm.convertRGB2YCC(rgb);
      
      // do grayscaling if desired
      if (this.doGrayScaling)
        {
         int cbValue = 128;
         int crValue = 128;
         
         for (int x = 0; x < ycc.getM2().size(); x++)
           {
            for (int y = 0; y < ycc.getM2().get(0).size(); y++)
              {
               ycc.getM2().get(x).set(y,cbValue);
               ycc.getM3().get(x).set(y,crValue);
              }
           }
        }
      
      // restore full cb cr resolution
      ycc = Util.upscaleColorResolution(ycc);
      // back to RGB
      MatrixTriple restoredRGB = Util.transformYCC2RGB(ycc);
      BufferedImage restoredImg = Util.createImageFromMatrixTriple(restoredRGB);

      //System.out.println("### Setting RGB image on canvas ###");
      this.currentImage = restoredImg;
     }
   catch (Exception e)
     {
      e.printStackTrace();
     }
   
   if (this.finalAction != null) this.finalAction.actionPerformed(new ActionEvent(this,1,"completed"));
  }
}