package de.mmt.mpeg.framework.process;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.swing.JFileChooser;

import de.mmt.mpeg.Exercise2;
import de.mmt.mpeg.framework.data.MacroBlock;
import de.mmt.mpeg.framework.data.MatrixTriple;
import de.mmt.mpeg.framework.util.BLKFilter;
import de.mmt.mpeg.framework.util.Util;

public class MacroblockProcess extends Thread
{
private boolean doPixeling = false;
private ActionListener finalAction = null;
private BufferedImage currentImage = null;

public MacroblockProcess(BufferedImage imageToConvert,ActionListener finalAction)
  {
   this(imageToConvert,finalAction,false);
  }

public MacroblockProcess(BufferedImage imageToConvert,ActionListener finalAction,boolean doPixeling)
  {
   this.finalAction = finalAction;
   this.currentImage = imageToConvert;
   this.doPixeling = doPixeling;
  }

public BufferedImage getCurrentImage()
  {
   return this.currentImage;
  }

@Override
public void run()
  {
   Exercise2 algorithm = new Exercise2();
   
   try
     {      
      MatrixTriple rgb = Util.getRGBMatricesFromImage(this.currentImage);
      MacroBlock[][] mbm = algorithm.generateMacroBlockMatrix(rgb);
      //System.out.println("### Computed macro blocks ###"); 
      
      JFileChooser jfc = new JFileChooser();
      jfc.setDialogTitle("Generierte Makroblockdatei speichern...");
      jfc.setFileFilter(new BLKFilter());
      int returnVal = JFileChooser.ERROR_OPTION;
      File storageFile = null;
      
      while (returnVal != JFileChooser.APPROVE_OPTION || storageFile == null || storageFile.isDirectory())
        {
         returnVal = jfc.showSaveDialog(null);
         
         // when user presses cancel, abort everything
         if (returnVal == JFileChooser.CANCEL_OPTION)
           {
            if (this.finalAction != null) this.finalAction.actionPerformed(new ActionEvent(this,1,"aborted"));
            return;
           }
         
         storageFile = jfc.getSelectedFile();
        }
      
      if (!storageFile.getName().toLowerCase().endsWith(".blk")) storageFile = new File(storageFile.getParent(),storageFile.getName()+".blk");
      
      // save macroblock file
      Util.exportBlockMatrix(mbm,storageFile.getPath());
      //System.out.println("### Saved macroblock file ###"); 
              
      // restore pixel matrix from macroblock
      MatrixTriple ycc = Util.convertToMatrixTriple(mbm,this.doPixeling);
            
      // back to RGB
      MatrixTriple restoredRGB = Util.transformYCC2RGB(ycc);
      BufferedImage restoredImg = Util.createImageFromMatrixTriple(restoredRGB);

      //System.out.println("### Setting RGB image on canvas ###");
      this.currentImage = restoredImg;
     }
   catch (Exception e)
     {
      e.printStackTrace();
      this.finalAction.actionPerformed(new ActionEvent(this,1,"error"));
     }
   
   if (this.finalAction != null) this.finalAction.actionPerformed(new ActionEvent(this,1,"completed"));
  }
}